import os
import sys
import pygame

os.environ["SDL_VIDEO_CENTERED"] = "1"
pygame.init()
clock = pygame.time.Clock()
size = width, height = 500, 800
screen = pygame.display.set_mode(size)
black = 0, 0, 0
background = pygame.image.load("background.png")
f = open("config.txt")
config = f.readlines()
f.close()
config[0] = int(config[0].split()[1])
config[1] = int(config[1].split()[1])


class Ball:
    sprite = pygame.image.load("ball.png")
    rect = sprite.get_rect()
    speed = [-config[0], -config[0]]  # [horizontal,vertical]

    def __init__(self, xy):
        self.rect.top = xy[1] - self.rect.height
        self.rect.left = xy[0]

    def basic_movement(self):
        self.rect = self.rect.move(self.speed)
        if self.rect.left < 0 or self.rect.right > width:  # left and right window collision ball
            self.speed[0] = -self.speed[0]
        if self.rect.top < 0 or self.rect.bottom > height:  # up and down window collision ball
            self.speed[1] = -self.speed[1]

    def blit(self):
        screen.blit(self.sprite, self.rect)

    def move(self, dx, dy, rect):

        # Move each axis separately. Note that this checks for collisions both times.
        if dx != 0:
            self.move_single_axis(dx, 0, rect)
        if dy != 0:
            self.move_single_axis(0, dy, rect)

        if self.rect.left < 0 or self.rect.right > width:  # left and right window collision ball
            self.speed[0] = -self.speed[0]
        if self.rect.top < 0 or self.rect.bottom > height:  # up and down window collision ball
            self.speed[1] = -self.speed[1]

    def move_single_axis(self, dx, dy, rect):

        # Move the rect
        self.rect.x += dx
        self.rect.y += dy

        if self.rect.colliderect(pad):
            rect=pad.rect
            
            if dx > 0:  # Moving right; Hit the left side
                self.rect.right = rect.left
                self.speed[0] = -self.speed[0]
                self.speed[1] = -self.speed[1]
            if dx < 0:  # Moving left; Hit the right side
                self.rect.left = rect.right
                self.speed[0] = -self.speed[0]
                self.speed[1] = -self.speed[1]
            if dy > 0:  # Moving down; Hit the top side
                self.rect.bottom = rect.top
                self.speed[1] = -self.speed[1]
            if dy < 0:  # Moving up; Hit the bottom side
                self.rect.top = rect.bottom
                self.speed[1] = -self.speed[1]

        for block in blocks:
            if self.rect.colliderect(block):
                if dx > 0:  # Moving right; Hit the left side
                    self.rect.right = block.rect.left
                    self.speed[0] = -self.speed[0]
                if dx < 0:  # Moving left; Hit the right side
                    self.rect.left = block.rect.right
                    self.speed[0] = -self.speed[0]
                if dy > 0:  # Moving down; Hit the top side
                    self.rect.bottom = block.rect.top
                    self.speed[1] = -self.speed[1]
                if dy < 0:  # Moving up; Hit the bottom side
                    self.rect.top = block.rect.bottom
                    self.speed[1] = -self.speed[1]
                blocks.remove(block)
        block = None


class Pad:
    sprite = pygame.image.load("pad.png")
    rect = sprite.get_rect()
    speed = [0, 0]
    movementspeed = config[1]

    def __init__(self, pos):
        self.rect.left = pos[0]
        self.rect.top = pos[1]

    def basic_movement(self, ballrect):

        self.speed = [0, 0]
        if (pygame.key.get_pressed()[pygame.K_RIGHT] != 0) and self.rect.left + self.rect.width < width - self.movementspeed:
            self.speed[0] += self.movementspeed
        if (pygame.key.get_pressed()[pygame.K_LEFT] != 0) and self.rect.left > self.movementspeed:
            self.speed[0] -= self.movementspeed

        self.rect = self.rect.move(self.speed)

        if self.rect.colliderect(ballrect):
            self.speed[0] = -self.speed[0]
            self.rect = self.rect.move(self.speed)

    def blit(self):
        screen.blit(self.sprite, self.rect)


class Block:
    sprite = pygame.image.load("block.png")

    def __init__(self, pos):
        blocks.append(self)
        self.rect = pygame.Rect(pos[0], pos[1], 24, 24)

    def blit(self):
        screen.blit(self.sprite, self.rect)


pad = Pad((width // 5, height // 1.2))
ball = Ball(pad.rect.midtop)

f = open('level.txt')
level = f.readlines()
f.close()

blocks = []

def initialize():
    pygame.display.set_caption("Use arrow keys to move the platform.")
    del blocks[:]
    ball.speed = [-config[0], -config[0]]
    ball.rect.top = pad.rect.midtop[1]  - ball.rect.height
    ball.rect.left = pad.rect.midtop[0]
    

    x = (width-len(level[0])*24)//2+12
    y = 100
    for row in level:
        for col in row:
            if col == "B":
                Block((x, y))
            x += 24
        y += 24
        x = (width-len(level[0])*24)//2+12

initialize()

while 1:
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            sys.exit()

    # pad movement
    pad.basic_movement(ball.rect)

    # ball movement with ball-pad collision
    ball.move(ball.speed[0], ball.speed[1], pad)
    if len(blocks) == 0:
        pygame.display.set_caption("You Win!")
    elif ball.rect.top > pad.rect.top:
        pygame.display.set_caption("You Lose. Press r to restart")
        blocks=[]
        blocks.append(Block((-50,-50)))
        

    if pygame.key.get_pressed()[pygame.K_BACKSLASH]:
        blocks = []

    if pygame.key.get_pressed()[pygame.K_r]:
        initialize()

    screen.fill(black)
    screen.blit(background, [0, 0])
    for block in blocks:
        block.blit()
    block = None
    pad.blit()
    ball.blit()
    pygame.display.flip()
    clock.tick(60)
